@@ -15,6 +15,8 @@ use crate::{
     indenter::indented,
     list::{BinaryList, OutputFormat, RustBuildMeta, Styles, TestListState},
     reuse_build::PathMapper,
+    run_mode::NextestRunMode,
+    runner::Interceptor,
     target_runner::{PlatformRunner, TargetRunner},
     test_command::{LocalExecuteContext, TestCommand, TestCommandPhase},
     test_filter::{BinaryMismatchReason, FilterBinaryMatch, FilterBound, TestFilterBuilder},
@@ -31,8 +33,8 @@ use iddqd::{IdOrdItem, IdOrdMap, id_upcast};
 use nextest_filtering::{BinaryQuery, EvalContext, TestQuery};
 use nextest_metadata::{
     BuildPlatform, FilterMatch, MismatchReason, RustBinaryId, RustNonTestBinaryKind,
-    RustTestBinaryKind, RustTestBinarySummary, RustTestCaseSummary, RustTestSuiteStatusSummary,
-    RustTestSuiteSummary, TestListSummary,
+    RustTestBinaryKind, RustTestBinarySummary, RustTestCaseSummary, RustTestKind,
+    RustTestSuiteStatusSummary, RustTestSuiteSummary, TestCaseName, TestListSummary,
 };
 use owo_colors::OwoColorize;
 use quick_junit::ReportUuid;
@@ -204,6 +206,11 @@ pub struct SkipCounts {
     /// The number of skipped tests.
     pub skipped_tests: usize,
 
+    /// The number of tests skipped because they are not benchmarks.
+    ///
+    /// This is used when running in benchmark mode.
+    pub skipped_tests_non_benchmark: usize,
+
     /// The number of tests skipped due to not being in the default set.
     pub skipped_tests_default_filter: usize,
 
@@ -218,6 +225,7 @@ pub struct SkipCounts {
 #[derive(Clone, Debug)]
 pub struct TestList<'g> {
     test_count: usize,
+    mode: NextestRunMode,
     rust_build_meta: RustBuildMeta<TestListState>,
     rust_suites: IdOrdMap<RustTestSuite<'g>>,
     workspace_root: Utf8PathBuf,
@@ -315,6 +323,7 @@ impl<'g> TestList<'g> {
 
         Ok(Self {
             rust_suites,
+            mode: filter.mode(),
             workspace_root,
             env,
             rust_build_meta,
@@ -373,6 +382,7 @@ impl<'g> TestList<'g> {
 
         Ok(Self {
             rust_suites,
+            mode: filter.mode(),
             workspace_root,
             env,
             rust_build_meta,
@@ -387,6 +397,11 @@ impl<'g> TestList<'g> {
         self.test_count
     }
 
+    /// Returns the mode nextest is running in.
+    pub fn mode(&self) -> NextestRunMode {
+        self.mode
+    }
+
     /// Returns the Rust build-related metadata for this test list.
     pub fn rust_build_meta(&self) -> &RustBuildMeta<TestListState> {
         &self.rust_build_meta
@@ -395,10 +410,17 @@ impl<'g> TestList<'g> {
     /// Returns the total number of skipped tests.
     pub fn skip_counts(&self) -> &SkipCounts {
         self.skip_counts.get_or_init(|| {
+            let mut skipped_tests_non_benchmark = 0;
             let mut skipped_tests_default_filter = 0;
             let skipped_tests = self
                 .iter_tests()
                 .filter(|instance| match instance.test_info.filter_match {
+                    FilterMatch::Mismatch {
+                        reason: MismatchReason::NotBenchmark,
+                    } => {
+                        skipped_tests_non_benchmark += 1;
+                        true
+                    }
                     FilterMatch::Mismatch {
                         reason: MismatchReason::DefaultFilter,
                     } => {
@@ -428,6 +450,7 @@ impl<'g> TestList<'g> {
 
             SkipCounts {
                 skipped_tests,
+                skipped_tests_non_benchmark,
                 skipped_tests_default_filter,
                 skipped_binaries,
                 skipped_binaries_default_filter,
@@ -508,6 +531,9 @@ impl<'g> TestList<'g> {
             OutputFormat::Human { verbose } => self
                 .write_human(writer, verbose, colorize)
                 .map_err(WriteTestListError::Io),
+            OutputFormat::Oneline { verbose } => self
+                .write_oneline(writer, verbose, colorize)
+                .map_err(WriteTestListError::Io),
             OutputFormat::Serializable(format) => format.to_writer(&self.to_summary(), writer),
         }
     }
@@ -517,13 +543,18 @@ impl<'g> TestList<'g> {
         self.rust_suites.iter()
     }
 
+    /// Looks up a test suite by binary ID.
+    pub fn get_suite(&self, binary_id: &RustBinaryId) -> Option<&RustTestSuite<'_>> {
+        self.rust_suites.get(binary_id)
+    }
+
     /// Iterates over the list of tests, returning the path and test name.
     pub fn iter_tests(&self) -> impl Iterator<Item = TestInstance<'_>> + '_ {
         self.rust_suites.iter().flat_map(|test_suite| {
             test_suite
                 .status
                 .test_cases()
-                .map(move |(name, test_info)| TestInstance::new(name, test_suite, test_info))
+                .map(move |case| TestInstance::new(case, test_suite))
         })
     }
 
@@ -551,6 +582,7 @@ impl<'g> TestList<'g> {
     pub(crate) fn empty() -> Self {
         Self {
             test_count: 0,
+            mode: NextestRunMode::Test,
             workspace_root: Utf8PathBuf::new(),
             rust_build_meta: RustBuildMeta::empty(),
             env: EnvironmentMap::empty(),
@@ -602,46 +634,42 @@ impl<'g> TestList<'g> {
         non_ignored: impl AsRef<str>,
         ignored: impl AsRef<str>,
     ) -> Result<RustTestSuite<'g>, CreateTestListError> {
-        let mut test_cases = BTreeMap::new();
+        let mut test_cases = IdOrdMap::new();
 
         // Treat ignored and non-ignored as separate sets of single filters, so that partitioning
         // based on one doesn't affect the other.
         let mut non_ignored_filter = filter.build();
-        for test_name in Self::parse(&test_binary.binary_id, non_ignored.as_ref())? {
-            test_cases.insert(
-                test_name.into(),
-                RustTestCaseSummary {
+        for (test_name, kind) in Self::parse(&test_binary.binary_id, non_ignored.as_ref())? {
+            let name = TestCaseName::new(test_name);
+            let filter_match =
+                non_ignored_filter.filter_match(&test_binary, &name, &kind, ecx, bound, false);
+            test_cases.insert_overwrite(RustTestCase {
+                name,
+                test_info: RustTestCaseSummary {
+                    kind: Some(kind),
                     ignored: false,
-                    filter_match: non_ignored_filter.filter_match(
-                        &test_binary,
-                        test_name,
-                        ecx,
-                        bound,
-                        false,
-                    ),
+                    filter_match,
                 },
-            );
+            });
         }
 
         let mut ignored_filter = filter.build();
-        for test_name in Self::parse(&test_binary.binary_id, ignored.as_ref())? {
+        for (test_name, kind) in Self::parse(&test_binary.binary_id, ignored.as_ref())? {
             // Note that libtest prints out:
             // * just ignored tests if --ignored is passed in
             // * all tests, both ignored and non-ignored, if --ignored is not passed in
             // Adding ignored tests after non-ignored ones makes everything resolve correctly.
-            test_cases.insert(
-                test_name.into(),
-                RustTestCaseSummary {
+            let name = TestCaseName::new(test_name);
+            let filter_match =
+                ignored_filter.filter_match(&test_binary, &name, &kind, ecx, bound, true);
+            test_cases.insert_overwrite(RustTestCase {
+                name,
+                test_info: RustTestCaseSummary {
+                    kind: Some(kind),
                     ignored: true,
-                    filter_match: ignored_filter.filter_match(
-                        &test_binary,
-                        test_name,
-                        ecx,
-                        bound,
-                        true,
-                    ),
+                    filter_match,
                 },
-            );
+            });
         }
 
         Ok(test_binary.into_test_suite(RustTestSuiteStatus::Listed {
@@ -660,36 +688,12 @@ impl<'g> TestList<'g> {
     fn parse<'a>(
         binary_id: &'a RustBinaryId,
         list_output: &'a str,
-    ) -> Result<Vec<&'a str>, CreateTestListError> {
-        let mut list = Self::parse_impl(binary_id, list_output).collect::<Result<Vec<_>, _>>()?;
+    ) -> Result<Vec<(&'a str, RustTestKind)>, CreateTestListError> {
+        let mut list = parse_list_lines(binary_id, list_output).collect::<Result<Vec<_>, _>>()?;
         list.sort_unstable();
         Ok(list)
     }
 
-    fn parse_impl<'a>(
-        binary_id: &'a RustBinaryId,
-        list_output: &'a str,
-    ) -> impl Iterator<Item = Result<&'a str, CreateTestListError>> + 'a + use<'a> {
-        // The output is in the form:
-        // <test name>: test
-        // <test name>: test
-        // ...
-
-        list_output.lines().map(move |line| {
-            line.strip_suffix(": test")
-                .or_else(|| line.strip_suffix(": benchmark"))
-                .ok_or_else(|| {
-                    CreateTestListError::parse_line(
-                        binary_id.clone(),
-                        format!(
-                            "line '{line}' did not end with the string ': test' or ': benchmark'"
-                        ),
-                        list_output,
-                    )
-                })
-        })
-    }
-
     /// Writes this test list out in a human-friendly format.
     pub fn write_human(
         &self,
@@ -738,7 +742,7 @@ impl<'g> TestList<'g> {
                 && info
                     .status
                     .test_cases()
-                    .all(|(_, test_case)| !test_case.filter_match.is_match())
+                    .all(|case| !case.test_info.filter_match.is_match())
             {
                 continue;
             }
@@ -766,19 +770,19 @@ impl<'g> TestList<'g> {
                 RustTestSuiteStatus::Listed { test_cases } => {
                     let matching_tests: Vec<_> = test_cases
                         .iter()
-                        .filter(|(name, _)| matcher.is_match(name))
+                        .filter(|case| matcher.is_match(&case.name))
                         .collect();
                     if matching_tests.is_empty() {
                         writeln!(indented, "(no tests)")?;
                     } else {
-                        for (name, info) in matching_tests {
-                            match (verbose, info.filter_match.is_match()) {
+                        for case in matching_tests {
+                            match (verbose, case.test_info.filter_match.is_match()) {
                                 (_, true) => {
-                                    write_test_name(name, &styles, &mut indented)?;
+                                    write_test_name(&case.name, &styles, &mut indented)?;
                                     writeln!(indented)?;
                                 }
                                 (true, false) => {
-                                    write_test_name(name, &styles, &mut indented)?;
+                                    write_test_name(&case.name, &styles, &mut indented)?;
                                     writeln!(indented, " (skipped)")?;
                                 }
                                 (false, false) => {
@@ -797,6 +801,83 @@ impl<'g> TestList<'g> {
         }
         Ok(())
     }
+
+    /// Writes this test list out in a one-line-per-test format.
+    pub fn write_oneline(
+        &self,
+        writer: &mut dyn WriteStr,
+        verbose: bool,
+        colorize: bool,
+    ) -> io::Result<()> {
+        let mut styles = Styles::default();
+        if colorize {
+            styles.colorize();
+        }
+
+        for info in &self.rust_suites {
+            match &info.status {
+                RustTestSuiteStatus::Listed { test_cases } => {
+                    for case in test_cases.iter() {
+                        let is_match = case.test_info.filter_match.is_match();
+                        // Skip tests that don't match the filter (unless verbose).
+                        if !verbose && !is_match {
+                            continue;
+                        }
+
+                        write!(writer, "{} ", info.binary_id.style(styles.binary_id))?;
+                        write_test_name(&case.name, &styles, writer)?;
+
+                        if verbose {
+                            write!(
+                                writer,
+                                " [{}{}] [{}{}] [{}{}]{}",
+                                "bin: ".style(styles.field),
+                                info.binary_path,
+                                "cwd: ".style(styles.field),
+                                info.cwd,
+                                "build platform: ".style(styles.field),
+                                info.build_platform,
+                                if is_match { "" } else { " (skipped)" },
+                            )?;
+                        }
+
+                        writeln!(writer)?;
+                    }
+                }
+                RustTestSuiteStatus::Skipped { .. } => {
+                    // Skip binaries that were not listed.
+                }
+            }
+        }
+
+        Ok(())
+    }
+}
+
+fn parse_list_lines<'a>(
+    binary_id: &'a RustBinaryId,
+    list_output: &'a str,
+) -> impl Iterator<Item = Result<(&'a str, RustTestKind), CreateTestListError>> + 'a + use<'a> {
+    // The output is in the form:
+    // <test name>: test
+    // <test name>: test
+    // ...
+
+    list_output
+        .lines()
+        .map(move |line| match line.strip_suffix(": test") {
+            Some(test_name) => Ok((test_name, RustTestKind::TEST)),
+            None => match line.strip_suffix(": benchmark") {
+                Some(test_name) => Ok((test_name, RustTestKind::BENCH)),
+                None => Err(CreateTestListError::parse_line(
+                    binary_id.clone(),
+                    format!(
+                        "line {line:?} did not end with the string \": test\" or \": benchmark\""
+                    ),
+                    list_output,
+                )),
+            },
+        })
 }
 
 /// Profile implementation for test lists.
@@ -825,10 +906,11 @@ pub struct TestPriorityQueue<'a> {
 
 impl<'a> TestPriorityQueue<'a> {
     fn new(test_list: &'a TestList<'a>, profile: &'a EvaluatableProfile<'a>) -> Self {
+        let mode = test_list.mode();
         let mut tests = test_list
             .iter_tests()
             .map(|instance| {
-                let settings = profile.settings_for(&instance.to_test_query());
+                let settings = profile.settings_for(mode, &instance.to_test_query());
                 TestInstanceWithSettings { instance, settings }
             })
             .collect::<Vec<_>>();
@@ -964,6 +1046,7 @@ impl RustTestArtifact<'_> {
             &self.cwd,
             &self.package,
             &self.non_test_binaries,
+            &Interceptor::None, // Interceptors are not used during the test list phase.
         );
 
         let output =
@@ -1002,7 +1085,7 @@ pub enum RustTestSuiteStatus {
     /// The test suite was executed with `--list` and the list of test cases was obtained.
     Listed {
         /// The test cases contained within this test suite.
-        test_cases: DebugIgnore<BTreeMap<String, RustTestCaseSummary>>,
+        test_cases: DebugIgnore<IdOrdMap<RustTestCase>>,
     },
 
     /// The test suite was not executed.
@@ -1012,7 +1095,7 @@ pub enum RustTestSuiteStatus {
     },
 }
 
-static EMPTY_TEST_CASE_MAP: BTreeMap<String, RustTestCaseSummary> = BTreeMap::new();
+static EMPTY_TEST_CASE_MAP: IdOrdMap<RustTestCase> = IdOrdMap::new();
 
 impl RustTestSuiteStatus {
     /// Returns the number of test cases within this suite.
@@ -1024,7 +1107,7 @@ impl RustTestSuiteStatus {
     }
 
     /// Returns the list of test cases within this suite.
-    pub fn test_cases(&self) -> impl Iterator<Item = (&str, &RustTestCaseSummary)> + '_ {
+    pub fn test_cases(&self) -> impl Iterator<Item = &RustTestCase> + '_ {
         match self {
             RustTestSuiteStatus::Listed { test_cases } => test_cases.iter(),
             RustTestSuiteStatus::Skipped { .. } => {
@@ -1032,7 +1115,6 @@ impl RustTestSuiteStatus {
                 EMPTY_TEST_CASE_MAP.iter()
             }
         }
-        .map(|(name, case)| (name.as_str(), case))
     }
 
     /// Converts this status to its serializable form.
@@ -1040,12 +1122,17 @@ impl RustTestSuiteStatus {
         &self,
     ) -> (
         RustTestSuiteStatusSummary,
-        BTreeMap<String, RustTestCaseSummary>,
+        BTreeMap<TestCaseName, RustTestCaseSummary>,
     ) {
         match self {
-            Self::Listed { test_cases } => {
-                (RustTestSuiteStatusSummary::LISTED, test_cases.clone().0)
-            }
+            Self::Listed { test_cases } => (
+                RustTestSuiteStatusSummary::LISTED,
+                test_cases
+                    .iter()
+                    .cloned()
+                    .map(|case| (case.name, case.test_info))
+                    .collect(),
+            ),
             Self::Skipped {
                 reason: BinaryMismatchReason::Expression,
             } => (RustTestSuiteStatusSummary::SKIPPED, BTreeMap::new()),
@@ -1059,11 +1146,29 @@ impl RustTestSuiteStatus {
     }
 }
 
+/// A single test case within a test suite.
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub struct RustTestCase {
+    /// The name of the test.
+    pub name: TestCaseName,
+
+    /// Information about the test.
+    pub test_info: RustTestCaseSummary,
+}
+
+impl IdOrdItem for RustTestCase {
+    type Key<'a> = &'a TestCaseName;
+    fn key(&self) -> Self::Key<'_> {
+        &self.name
+    }
+    id_upcast!();
+}
+
 /// Represents a single test with its associated binary.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct TestInstance<'a> {
     /// The name of the test.
-    pub name: &'a str,
+    pub name: &'a TestCaseName,
 
     /// Information about the test suite.
     pub suite_info: &'a RustTestSuite<'a>,
@@ -1074,15 +1179,11 @@ pub struct TestInstance<'a> {
 
 impl<'a> TestInstance<'a> {
     /// Creates a new `TestInstance`.
-    pub(crate) fn new(
-        name: &'a (impl AsRef<str> + ?Sized),
-        suite_info: &'a RustTestSuite,
-        test_info: &'a RustTestCaseSummary,
-    ) -> Self {
+    pub(crate) fn new(case: &'a RustTestCase, suite_info: &'a RustTestSuite) -> Self {
         Self {
-            name: name.as_ref(),
+            name: &case.name,
             suite_info,
-            test_info,
+            test_info: &case.test_info,
         }
     }
 
@@ -1115,29 +1216,12 @@ impl<'a> TestInstance<'a> {
         &self,
         ctx: &TestExecuteContext<'_>,
         test_list: &TestList<'_>,
-        wrapper_script: Option<&'a WrapperScriptConfig>,
+        wrapper_script: Option<&WrapperScriptConfig>,
         extra_args: &[String],
+        interceptor: &Interceptor,
     ) -> TestCommand {
         // TODO: non-rust tests
-
-        let platform_runner = ctx
-            .target_runner
-            .for_build_platform(self.suite_info.build_platform);
-
-        let mut cli = TestCommandCli::default();
-        cli.apply_wrappers(
-            wrapper_script,
-            platform_runner,
-            test_list.workspace_root(),
-            &test_list.rust_build_meta().target_directory,
-        );
-        cli.push(self.suite_info.binary_path.as_str());
-
-        cli.extend(["--exact", self.name, "--nocapture"]);
-        if self.test_info.ignored {
-            cli.push("--ignored");
-        }
-        cli.extend(extra_args.iter().map(String::as_str));
+        let cli = self.compute_cli(ctx, test_list, wrapper_script, extra_args);
 
         let lctx = LocalExecuteContext {
             phase: TestCommandPhase::Run,
@@ -1158,8 +1242,55 @@ impl<'a> TestInstance<'a> {
             &self.suite_info.cwd,
             &self.suite_info.package,
             &self.suite_info.non_test_binaries,
+            interceptor,
         )
     }
+
+    pub(crate) fn command_line(
+        &self,
+        ctx: &TestExecuteContext<'_>,
+        test_list: &TestList<'_>,
+        wrapper_script: Option<&WrapperScriptConfig>,
+        extra_args: &[String],
+    ) -> Vec<String> {
+        self.compute_cli(ctx, test_list, wrapper_script, extra_args)
+            .to_owned_cli()
+    }
+
+    fn compute_cli(
+        &self,
+        ctx: &'a TestExecuteContext<'_>,
+        test_list: &TestList<'_>,
+        wrapper_script: Option<&'a WrapperScriptConfig>,
+        extra_args: &'a [String],
+    ) -> TestCommandCli<'a> {
+        let platform_runner = ctx
+            .target_runner
+            .for_build_platform(self.suite_info.build_platform);
+
+        let mut cli = TestCommandCli::default();
+        cli.apply_wrappers(
+            wrapper_script,
+            platform_runner,
+            test_list.workspace_root(),
+            &test_list.rust_build_meta().target_directory,
+        );
+        cli.push(self.suite_info.binary_path.as_str());
+
+        cli.extend(["--exact", self.name.as_str(), "--nocapture"]);
+        if self.test_info.ignored {
+            cli.push("--ignored");
+        }
+        match test_list.mode() {
+            NextestRunMode::Test => {}
+            NextestRunMode::Benchmark => {
+                cli.push("--bench");
+            }
+        }
+        cli.extend(extra_args.iter().map(String::as_str));
+
+        cli
+    }
 }
 
 #[derive(Clone, Debug, Default)]
@@ -1252,7 +1383,7 @@ pub struct TestInstanceId<'a> {
     pub binary_id: &'a RustBinaryId,
 
     /// The name of the test.
-    pub test_name: &'a str,
+    pub test_name: &'a TestCaseName,
 }
 
 impl TestInstanceId<'_> {
@@ -1285,6 +1416,36 @@ impl fmt::Display for TestInstanceId<'_> {
     }
 }
 
+/// An owned version of [`TestInstanceId`].
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub struct OwnedTestInstanceId {
+    /// The binary ID.
+    pub binary_id: RustBinaryId,
+
+    /// The name of the test.
+    pub test_name: TestCaseName,
+}
+
+impl OwnedTestInstanceId {
+    /// Borrow this as a [`TestInstanceId`].
+    pub fn as_ref(&self) -> TestInstanceId<'_> {
+        TestInstanceId {
+            binary_id: &self.binary_id,
+            test_name: &self.test_name,
+        }
+    }
+}
+
+impl TestInstanceId<'_> {
+    /// Convert this to an owned version.
+    pub fn to_owned(&self) -> OwnedTestInstanceId {
+        OwnedTestInstanceId {
+            binary_id: self.binary_id.clone(),
+            test_name: self.test_name.clone(),
+        }
+    }
+}
+
 /// Context required for test execution.
 #[derive(Clone, Debug)]
 pub struct TestExecuteContext<'a> {
@@ -1312,9 +1473,8 @@ mod tests {
     use guppy::CargoMetadata;
     use iddqd::id_ord_map;
     use indoc::indoc;
-    use maplit::btreemap;
     use nextest_filtering::{CompiledExpr, Filterset, FiltersetKind, ParseContext};
-    use nextest_metadata::{FilterMatch, MismatchReason, PlatformLibdirUnavailable};
+    use nextest_metadata::{FilterMatch, MismatchReason, PlatformLibdirUnavailable, RustTestKind};
     use pretty_assertions::assert_eq;
     use std::sync::LazyLock;
     use target_spec::Platform;
@@ -1336,6 +1496,7 @@ mod tests {
         let cx = ParseContext::new(&PACKAGE_GRAPH_FIXTURE);
 
         let test_filter = TestFilterBuilder::new(
+            NextestRunMode::Test,
             RunIgnored::Default,
             None,
             TestFilterPatterns::default(),
@@ -1423,30 +1584,54 @@ mod tests {
             id_ord_map! {
                 RustTestSuite {
                     status: RustTestSuiteStatus::Listed {
-                        test_cases: btreemap! {
-                            "tests::foo::test_bar".to_owned() => RustTestCaseSummary {
-                                ignored: false,
-                                filter_match: FilterMatch::Matches,
+                        test_cases: id_ord_map! {
+                            RustTestCase {
+                                name: TestCaseName::new("tests::foo::test_bar"),
+                                test_info: RustTestCaseSummary {
+                                    kind: Some(RustTestKind::TEST),
+                                    ignored: false,
+                                    filter_match: FilterMatch::Matches,
+                                },
                             },
-                            "tests::baz::test_quux".to_owned() => RustTestCaseSummary {
-                                ignored: false,
-                                filter_match: FilterMatch::Matches,
+                            RustTestCase {
+                                name: TestCaseName::new("tests::baz::test_quux"),
+                                test_info: RustTestCaseSummary {
+                                    kind: Some(RustTestKind::TEST),
+                                    ignored: false,
+                                    filter_match: FilterMatch::Matches,
+                                },
                             },
-                            "benches::bench_foo".to_owned() => RustTestCaseSummary {
-                                ignored: false,
-                                filter_match: FilterMatch::Matches,
+                            RustTestCase {
+                                name: TestCaseName::new("benches::bench_foo"),
+                                test_info: RustTestCaseSummary {
+                                    kind: Some(RustTestKind::BENCH),
+                                    ignored: false,
+                                    filter_match: FilterMatch::Matches,
+                                },
                             },
-                            "tests::ignored::test_bar".to_owned() => RustTestCaseSummary {
-                                ignored: true,
-                                filter_match: FilterMatch::Mismatch { reason: MismatchReason::Ignored },
+                            RustTestCase {
+                                name: TestCaseName::new("tests::ignored::test_bar"),
+                                test_info: RustTestCaseSummary {
+                                    kind: Some(RustTestKind::TEST),
+                                    ignored: true,
+                                    filter_match: FilterMatch::Mismatch { reason: MismatchReason::Ignored },
+                                },
                             },
-                            "tests::baz::test_ignored".to_owned() => RustTestCaseSummary {
-                                ignored: true,
-                                filter_match: FilterMatch::Mismatch { reason: MismatchReason::Ignored },
+                            RustTestCase {
+                                name: TestCaseName::new("tests::baz::test_ignored"),
+                                test_info: RustTestCaseSummary {
+                                    kind: Some(RustTestKind::TEST),
+                                    ignored: true,
+                                    filter_match: FilterMatch::Mismatch { reason: MismatchReason::Ignored },
+                                },
                             },
-                            "benches::ignored_bench_foo".to_owned() => RustTestCaseSummary {
-                                ignored: true,
-                                filter_match: FilterMatch::Mismatch { reason: MismatchReason::Ignored },
+                            RustTestCase {
+                                name: TestCaseName::new("benches::ignored_bench_foo"),
+                                test_info: RustTestCaseSummary {
+                                    kind: Some(RustTestKind::BENCH),
+                                    ignored: true,
+                                    filter_match: FilterMatch::Mismatch { reason: MismatchReason::Ignored },
+                                },
                             },
                         }.into(),
                     },
@@ -1553,12 +1738,14 @@ mod tests {
                   "status": "listed",
                   "testcases": {
                     "benches::bench_foo": {
+                      "kind": "bench",
                       "ignored": false,
                       "filter-match": {
                         "status": "matches"
                       }
                     },
                     "benches::ignored_bench_foo": {
+                      "kind": "bench",
                       "ignored": true,
                       "filter-match": {
                         "status": "mismatch",
@@ -1566,6 +1753,7 @@ mod tests {
                       }
                     },
                     "tests::baz::test_ignored": {
+                      "kind": "test",
                       "ignored": true,
                       "filter-match": {
                         "status": "mismatch",
@@ -1573,18 +1761,21 @@ mod tests {
                       }
                     },
                     "tests::baz::test_quux": {
+                      "kind": "test",
                       "ignored": false,
                       "filter-match": {
                         "status": "matches"
                       }
                     },
                     "tests::foo::test_bar": {
+                      "kind": "test",
                       "ignored": false,
                       "filter-match": {
                         "status": "matches"
                       }
                     },
                     "tests::ignored::test_bar": {
+                      "kind": "test",
                       "ignored": true,
                       "filter-match": {
                         "status": "mismatch",
@@ -1607,6 +1798,19 @@ mod tests {
                 }
               }
             }"#};
+        static EXPECTED_ONELINE: &str = indoc! {"
+            fake-package::fake-binary benches::bench_foo
+            fake-package::fake-binary tests::baz::test_quux
+            fake-package::fake-binary tests::foo::test_bar
+        "};
+        static EXPECTED_ONELINE_VERBOSE: &str = indoc! {"
+            fake-package::fake-binary benches::bench_foo [bin: /fake/binary] [cwd: /fake/cwd] [build platform: target]
+            fake-package::fake-binary benches::ignored_bench_foo [bin: /fake/binary] [cwd: /fake/cwd] [build platform: target] (skipped)
+            fake-package::fake-binary tests::baz::test_ignored [bin: /fake/binary] [cwd: /fake/cwd] [build platform: target] (skipped)
+            fake-package::fake-binary tests::baz::test_quux [bin: /fake/binary] [cwd: /fake/cwd] [build platform: target]
+            fake-package::fake-binary tests::foo::test_bar [bin: /fake/binary] [cwd: /fake/cwd] [build platform: target]
+            fake-package::fake-binary tests::ignored::test_bar [bin: /fake/binary] [cwd: /fake/cwd] [build platform: target] (skipped)
+        "};
 
         assert_eq!(
             test_list
@@ -1632,6 +1836,18 @@ mod tests {
                 .expect("json-pretty succeeded"),
             EXPECTED_JSON_PRETTY
         );
+        assert_eq!(
+            test_list
+                .to_string(OutputFormat::Oneline { verbose: false })
+                .expect("oneline succeeded"),
+            EXPECTED_ONELINE
+        );
+        assert_eq!(
+            test_list
+                .to_string(OutputFormat::Oneline { verbose: true })
+                .expect("oneline verbose succeeded"),
+            EXPECTED_ONELINE_VERBOSE
+        );
     }
 
     #[test]
@@ -1910,4 +2126,92 @@ mod tests {
             .metadata(&PackageId::new(PACKAGE_METADATA_ID))
             .expect("package ID is valid")
     }
+
+    #[test]
+    fn test_parse_list_lines() {
+        let binary_id = RustBinaryId::new("test-package::test-binary");
+
+        // Valid: tests only.
+        let input = indoc! {"
+            simple_test: test
+            module::nested_test: test
+            deeply::nested::module::test_name: test
+        "};
+        let results: Vec<_> = parse_list_lines(&binary_id, input)
+            .collect::<Result<_, _>>()
+            .expect("parsed valid test output");
+        insta::assert_debug_snapshot!("valid_tests", results);
+
+        // Valid: benchmarks only.
+        let input = indoc! {"
+            simple_bench: benchmark
+            benches::module::my_benchmark: benchmark
+        "};
+        let results: Vec<_> = parse_list_lines(&binary_id, input)
+            .collect::<Result<_, _>>()
+            .expect("parsed valid benchmark output");
+        insta::assert_debug_snapshot!("valid_benchmarks", results);
+
+        // Valid: mixed tests and benchmarks.
+        let input = indoc! {"
+            test_one: test
+            bench_one: benchmark
+            test_two: test
+            bench_two: benchmark
+        "};
+        let results: Vec<_> = parse_list_lines(&binary_id, input)
+            .collect::<Result<_, _>>()
+            .expect("parsed mixed output");
+        insta::assert_debug_snapshot!("mixed_tests_and_benchmarks", results);
+
+        // Valid: special characters.
+        let input = indoc! {r#"
+            test_with_underscore_123: test
+            test::with::colons: test
+            test_with_numbers_42: test
+        "#};
+        let results: Vec<_> = parse_list_lines(&binary_id, input)
+            .collect::<Result<_, _>>()
+            .expect("parsed tests with special characters");
+        insta::assert_debug_snapshot!("special_characters", results);
+
+        // Valid: empty input.
+        let input = "";
+        let results: Vec<_> = parse_list_lines(&binary_id, input)
+            .collect::<Result<_, _>>()
+            .expect("parsed empty output");
+        insta::assert_debug_snapshot!("empty_input", results);
+
+        // Invalid: wrong suffix.
+        let input = "invalid_test: wrong_suffix";
+        let result = parse_list_lines(&binary_id, input).collect::<Result<Vec<_>, _>>();
+        assert!(result.is_err());
+        insta::assert_snapshot!("invalid_suffix_error", result.unwrap_err());
+
+        // Invalid: missing suffix.
+        let input = "test_without_suffix";
+        let result = parse_list_lines(&binary_id, input).collect::<Result<Vec<_>, _>>();
+        assert!(result.is_err());
+        insta::assert_snapshot!("missing_suffix_error", result.unwrap_err());
+
+        // Invalid: partial valid (stops at first error).
+        let input = indoc! {"
+            valid_test: test
+            invalid_line
+            another_valid: benchmark
+        "};
+        let result = parse_list_lines(&binary_id, input).collect::<Result<Vec<_>, _>>();
+        assert!(result.is_err());
+        insta::assert_snapshot!("partial_valid_error", result.unwrap_err());
+
+        // Invalid: control character.
+        let input = indoc! {"
+            valid_test: test
+            \rinvalid_line
+            another_valid: benchmark
+        "};
+        let result = parse_list_lines(&binary_id, input).collect::<Result<Vec<_>, _>>();
+        assert!(result.is_err());
+        insta::assert_snapshot!("control_character_error", result.unwrap_err());
+    }
 }
