@@ -5,19 +5,21 @@
 
 use crate::{
     config::{
-        elements::{JunitConfig, LeakTimeoutResult},
+        elements::{JunitConfig, LeakTimeoutResult, SlowTimeoutResult},
         scripts::ScriptId,
     },
     errors::{DisplayErrorChain, WriteEventError},
     list::TestInstanceId,
     reporter::{
         UnitErrorDescription,
+        displayer::DisplayUnitKind,
         events::{
-            ExecutionDescription, ExecutionResult, FailureStatus, StressIndex, TestEvent,
-            TestEventKind, UnitKind,
+            ChildExecutionOutputDescription, ExecutionDescription, ExecutionResultDescription,
+            FailureDescription, StressIndex, TestEvent, TestEventKind, UnitKind,
         },
     },
-    test_output::{ChildExecutionOutput, ChildOutput},
+    run_mode::NextestRunMode,
+    test_output::ChildOutput,
 };
 use debug_ignore::DebugIgnore;
 use indexmap::IndexMap;
@@ -34,13 +36,15 @@ static PROCESS_FAILED_TO_START: &str = "(process failed to start)";
 
 #[derive(Clone, Debug)]
 pub(super) struct MetadataJunit<'cfg> {
+    mode: NextestRunMode,
     config: JunitConfig<'cfg>,
     test_suites: DebugIgnore<IndexMap<SuiteKey<'cfg>, TestSuite>>,
 }
 
 impl<'cfg> MetadataJunit<'cfg> {
-    pub(super) fn new(config: JunitConfig<'cfg>) -> Self {
+    pub(super) fn new(mode: NextestRunMode, config: JunitConfig<'cfg>) -> Self {
         Self {
+            mode,
             config,
             test_suites: DebugIgnore(IndexMap::new()),
         }
@@ -50,6 +54,8 @@ impl<'cfg> MetadataJunit<'cfg> {
         &mut self,
         event: Box<TestEvent<'cfg>>,
     ) -> Result<(), WriteEventError> {
+        // Copy mode at the start to avoid borrow checker conflicts.
+        let mode = self.mode;
         match event.kind {
             TestEventKind::RunStarted { .. }
             | TestEventKind::StressSubRunStarted { .. }
@@ -75,7 +81,8 @@ impl<'cfg> MetadataJunit<'cfg> {
                 let testcase_status = if is_success {
                     TestCaseStatus::success()
                 } else {
-                    let (kind, ty) = non_success_kind_and_type(UnitKind::Script, run_status.result);
+                    let (kind, ty) =
+                        non_success_kind_and_type(mode, UnitKind::Script, &run_status.result);
                     let mut testcase_status = TestCaseStatus::non_success(kind);
                     testcase_status.set_type(ty);
                     testcase_status
@@ -130,7 +137,7 @@ impl<'cfg> MetadataJunit<'cfg> {
                 junit_store_failure_output,
                 ..
             } => {
-                let testsuite = self.testsuite_for_test(stress_index, test_instance.id());
+                let testsuite = self.testsuite_for_test(stress_index, test_instance);
 
                 let (mut testcase_status, main_status, reruns) = match run_statuses.describe() {
                     ExecutionDescription::Success { single_status } => {
@@ -146,7 +153,7 @@ impl<'cfg> MetadataJunit<'cfg> {
                         ..
                     } => {
                         let (kind, ty) =
-                            non_success_kind_and_type(UnitKind::Test, first_status.result);
+                            non_success_kind_and_type(mode, UnitKind::Test, &first_status.result);
                         let mut testcase_status = TestCaseStatus::non_success(kind);
                         testcase_status.set_type(ty);
                         (testcase_status, first_status, retries)
@@ -154,7 +161,7 @@ impl<'cfg> MetadataJunit<'cfg> {
                 };
 
                 for rerun in reruns {
-                    let (kind, ty) = non_success_kind_and_type(UnitKind::Test, rerun.result);
+                    let (kind, ty) = non_success_kind_and_type(mode, UnitKind::Test, &rerun.result);
                     let mut test_rerun = TestRerun::new(kind);
                     test_rerun
                         .set_timestamp(rerun.start_time)
@@ -171,9 +178,9 @@ impl<'cfg> MetadataJunit<'cfg> {
                     testcase_status.add_rerun(test_rerun);
                 }
 
-                let mut testcase = TestCase::new(test_instance.name, testcase_status);
+                let mut testcase = TestCase::new(test_instance.test_name.as_str(), testcase_status);
                 testcase
-                    .set_classname(test_instance.suite_info.binary_id.as_str())
+                    .set_classname(test_instance.binary_id.as_str())
                     .set_timestamp(main_status.start_time)
                     .set_time(main_status.time_taken);
 
@@ -312,48 +319,80 @@ impl fmt::Display for SuiteKey<'_> {
     }
 }
 
-fn non_success_kind_and_type(kind: UnitKind, result: ExecutionResult) -> (NonSuccessKind, String) {
+fn non_success_kind_and_type(
+    mode: NextestRunMode,
+    kind: UnitKind,
+    result: &ExecutionResultDescription,
+) -> (NonSuccessKind, String) {
     match result {
-        ExecutionResult::Fail {
-            failure_status: FailureStatus::Abort(_),
+        ExecutionResultDescription::Fail {
+            failure: FailureDescription::Abort { .. },
             leaked: true,
         } => (
             NonSuccessKind::Failure,
-            format!("{kind} abort with leaked handles"),
+            format!(
+                "{} abort with leaked handles",
+                DisplayUnitKind::new(mode, kind),
+            ),
         ),
-        ExecutionResult::Fail {
-            failure_status: FailureStatus::Abort(_),
+        ExecutionResultDescription::Fail {
+            failure: FailureDescription::Abort { .. },
             leaked: false,
-        } => (NonSuccessKind::Failure, format!("{kind} abort")),
-        ExecutionResult::Fail {
-            failure_status: FailureStatus::ExitCode(code),
+        } => (
+            NonSuccessKind::Failure,
+            format!("{} abort", DisplayUnitKind::new(mode, kind)),
+        ),
+        ExecutionResultDescription::Fail {
+            failure: FailureDescription::ExitCode { code },
             leaked: true,
         } => (
             NonSuccessKind::Failure,
-            format!("{kind} failure with exit code {code}, and leaked handles"),
+            format!(
+                "{} failure with exit code {code}, and leaked handles",
+                DisplayUnitKind::new(mode, kind),
+            ),
         ),
-        ExecutionResult::Fail {
-            failure_status: FailureStatus::ExitCode(code),
+        ExecutionResultDescription::Fail {
+            failure: FailureDescription::ExitCode { code },
             leaked: false,
         } => (
             NonSuccessKind::Failure,
-            format!("{kind} failure with exit code {code}"),
+            format!(
+                "{} failure with exit code {code}",
+                DisplayUnitKind::new(mode, kind),
+            ),
         ),
-        ExecutionResult::Timeout => (NonSuccessKind::Failure, format!("{kind} timeout")),
-        ExecutionResult::ExecFail => (NonSuccessKind::Error, "execution failure".to_owned()),
-        ExecutionResult::Leak {
+        ExecutionResultDescription::Timeout {
+            result: SlowTimeoutResult::Fail,
+        } => (
+            NonSuccessKind::Failure,
+            format!("{} timeout", DisplayUnitKind::new(mode, kind)),
+        ),
+        ExecutionResultDescription::ExecFail => {
+            (NonSuccessKind::Error, "execution failure".to_owned())
+        }
+        ExecutionResultDescription::Leak {
             result: LeakTimeoutResult::Pass,
         } => (
             NonSuccessKind::Error,
-            format!("{kind} passed but leaked handles"),
+            format!(
+                "{} passed but leaked handles",
+                DisplayUnitKind::new(mode, kind),
+            ),
         ),
-        ExecutionResult::Leak {
+        ExecutionResultDescription::Leak {
             result: LeakTimeoutResult::Fail,
         } => (
             NonSuccessKind::Error,
-            format!("{kind} exited with code 0, but leaked handles so was marked failed"),
+            format!(
+                "{} exited with code 0, but leaked handles so was marked failed",
+                DisplayUnitKind::new(mode, kind),
+            ),
         ),
-        ExecutionResult::Pass => {
+        ExecutionResultDescription::Pass
+        | ExecutionResultDescription::Timeout {
+            result: SlowTimeoutResult::Pass,
+        } => {
             unreachable!("this is a failure status")
         }
     }
@@ -415,7 +454,7 @@ impl TestcaseOrRerun<'_> {
 }
 
 fn set_execute_status_props(
-    exec_output: &ChildExecutionOutput,
+    exec_output: &ChildExecutionOutputDescription,
     store_stdout_stderr: bool,
     mut out: TestcaseOrRerun<'_>,
 ) {
@@ -428,7 +467,7 @@ fn set_execute_status_props(
 
     if store_stdout_stderr {
         match exec_output {
-            ChildExecutionOutput::Output {
+            ChildExecutionOutputDescription::Output {
                 output: ChildOutput::Split(split),
                 ..
             } => {
@@ -443,14 +482,14 @@ fn set_execute_status_props(
                     out.set_system_err(STDERR_NOT_CAPTURED);
                 }
             }
-            ChildExecutionOutput::Output {
+            ChildExecutionOutputDescription::Output {
                 output: ChildOutput::Combined { output },
                 ..
             } => {
                 out.set_system_out(output.as_str_lossy())
                     .set_system_err(STDOUT_STDERR_COMBINED);
             }
-            ChildExecutionOutput::StartError(_) => {
+            ChildExecutionOutputDescription::StartError(_) => {
                 out.set_system_out(PROCESS_FAILED_TO_START)
                     .set_system_err(PROCESS_FAILED_TO_START);
             }
@@ -462,10 +501,11 @@ fn set_execute_status_props(
 mod tests {
     use super::*;
     #[cfg(unix)]
-    use crate::reporter::events::AbortStatus;
+    use crate::reporter::events::{AbortStatus, SIGTERM};
     use crate::{
         errors::{ChildError, ChildFdError, ChildStartError, ErrorList},
-        test_output::ChildSplitOutput,
+        reporter::events::{ChildExecutionOutputDescription, ExecutionResult, FailureStatus},
+        test_output::{ChildExecutionOutput, ChildSplitOutput},
     };
     use bytes::Bytes;
     use std::{io, sync::Arc};
@@ -482,7 +522,8 @@ mod tests {
                         output: Bytes::from("stdout\nstderr").into(),
                     },
                     errors: None,
-                },
+                }
+                .into(),
                 store_stdout_stderr: true,
                 message: None,
                 description: None,
@@ -498,7 +539,8 @@ mod tests {
                         output: Bytes::from("stdout\nstderr").into(),
                     },
                     errors: None,
-                },
+                }
+                .into(),
                 store_stdout_stderr: false,
                 message: None,
                 description: None,
@@ -515,7 +557,8 @@ mod tests {
                         stderr: Some(Bytes::from("stderr").into()),
                     }),
                     errors: None,
-                },
+                }
+                .into(),
                 store_stdout_stderr: true,
                 message: None,
                 description: None,
@@ -540,7 +583,8 @@ mod tests {
                         .into(),
                     },
                     errors: None,
-                },
+                }
+                .into(),
                 store_stdout_stderr: true,
                 message: Some("thread 'foo' panicked at xyz.rs:40"),
                 description: Some("thread 'foo' panicked at xyz.rs:40:\nstrange\nextra\nextra2"),
@@ -569,7 +613,8 @@ mod tests {
                         ),
                     }),
                     errors: None,
-                },
+                }
+                .into(),
                 store_stdout_stderr: false,
                 message: Some("thread 'foo' panicked at xyz.rs:40"),
                 description: Some(
@@ -585,9 +630,7 @@ mod tests {
                 status: TestCaseStatus::non_success(NonSuccessKind::Failure),
                 output: ChildExecutionOutput::Output {
                     result: Some(ExecutionResult::Fail {
-                        failure_status: FailureStatus::Abort(AbortStatus::UnixSignal(
-                            libc::SIGTERM,
-                        )),
+                        failure_status: FailureStatus::Abort(AbortStatus::UnixSignal(SIGTERM)),
                         leaked: false,
                     }),
                     output: ChildOutput::Split(ChildSplitOutput {
@@ -595,7 +638,8 @@ mod tests {
                         stderr: None,
                     }),
                     errors: None,
-                },
+                }
+                .into(),
                 store_stdout_stderr: true,
                 message: Some("process aborted with signal 15 (SIGTERM)"),
                 description: Some("process aborted with signal 15 (SIGTERM)"),
@@ -608,9 +652,7 @@ mod tests {
                 status: TestCaseStatus::non_success(NonSuccessKind::Failure),
                 output: ChildExecutionOutput::Output {
                     result: Some(ExecutionResult::Fail {
-                        failure_status: FailureStatus::Abort(AbortStatus::UnixSignal(
-                            libc::SIGTERM,
-                        )),
+                        failure_status: FailureStatus::Abort(AbortStatus::UnixSignal(SIGTERM)),
                         leaked: true,
                     }),
                     output: ChildOutput::Split(ChildSplitOutput {
@@ -625,7 +667,8 @@ mod tests {
                             io::Error::other("huh"),
                         )))],
                     ),
-                },
+                }
+                .into(),
                 store_stdout_stderr: false,
                 message: Some("3 errors occurred executing test"),
                 description: Some(indoc::indoc! {"
@@ -659,7 +702,8 @@ mod tests {
                             io::Error::other("stdout error"),
                         )))],
                     ),
-                },
+                }
+                .into(),
                 store_stdout_stderr: false,
                 message: Some("2 errors occurred executing test"),
                 description: Some(indoc::indoc! {"
@@ -677,7 +721,8 @@ mod tests {
                 status: TestCaseStatus::non_success(NonSuccessKind::Error),
                 output: ChildExecutionOutput::StartError(ChildStartError::Spawn(Arc::new(
                     io::Error::other("start error"),
-                ))),
+                )))
+                .into(),
                 store_stdout_stderr: true,
                 message: Some("error spawning child process"),
                 description: Some(indoc::indoc! {"
@@ -726,7 +771,7 @@ mod tests {
     struct ExecuteStatusPropsCase<'a> {
         comment: &'a str,
         status: TestCaseStatus,
-        output: ChildExecutionOutput,
+        output: ChildExecutionOutputDescription,
         store_stdout_stderr: bool,
         message: Option<&'a str>,
         description: Option<&'a str>,
