@@ -1,12 +1,11 @@
 // Copyright (c) The nextest Contributors
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-use super::events::{AbortStatus, ExecutionResult, FailureStatus, UnitKind};
-use crate::{
-    errors::{ChildError, ChildStartError, ErrorList},
-    helpers::display_abort_status,
-    test_output::{ChildExecutionOutput, ChildOutput},
+use super::events::{
+    AbortDescription, ChildErrorDescription, ChildExecutionOutputDescription,
+    ChildStartErrorDescription, ExecutionResultDescription, FailureDescription, UnitKind,
 };
+use crate::{errors::ErrorList, test_output::ChildOutput};
 use bstr::ByteSlice;
 use regex::bytes::{Regex, RegexBuilder};
 use std::{fmt, sync::LazyLock};
@@ -17,25 +16,25 @@ use thiserror::Error;
 #[derive(Clone, Debug)]
 pub struct UnitErrorDescription<'a> {
     kind: UnitKind,
-    start_error: Option<&'a ChildStartError>,
-    output_errors: Option<&'a ErrorList<ChildError>>,
+    start_error: Option<&'a ChildStartErrorDescription>,
+    output_errors: Option<&'a ErrorList<ChildErrorDescription>>,
     abort: Option<UnitAbortDescription>,
     output_slice: Option<TestOutputErrorSlice<'a>>,
 }
 
 impl<'a> UnitErrorDescription<'a> {
     /// Adds the execution output of a child process to the description.
-    pub fn new(kind: UnitKind, output: &'a ChildExecutionOutput) -> Self {
+    pub fn new(kind: UnitKind, output: &'a ChildExecutionOutputDescription) -> Self {
         let mut start_error = None;
         let mut output_errors = None;
         let mut abort = None;
         let mut output_slice = None;
 
         match output {
-            ChildExecutionOutput::StartError(error) => {
+            ChildExecutionOutputDescription::StartError(error) => {
                 start_error = Some(error);
             }
-            ChildExecutionOutput::Output {
+            ChildExecutionOutputDescription::Output {
                 result,
                 output,
                 errors,
@@ -61,13 +60,14 @@ impl<'a> UnitErrorDescription<'a> {
                         }
                     }
 
-                    if let ExecutionResult::Fail {
-                        failure_status: FailureStatus::Abort(status),
+                    if let ExecutionResultDescription::Fail {
+                        failure: FailureDescription::Abort { abort: status },
                         leaked,
+                        ..
                     } = result
                     {
                         abort = Some(UnitAbortDescription {
-                            status: *status,
+                            description: status.clone(),
                             leaked: *leaked,
                         });
                     }
@@ -120,12 +120,10 @@ impl<'a> UnitErrorDescription<'a> {
 
     fn exec_fail_errors(&self) -> impl Iterator<Item = &dyn std::error::Error> {
         self.start_error
-            .as_ref()
             .map(|error| error as &dyn std::error::Error)
             .into_iter()
             .chain(
                 self.output_errors
-                    .as_ref()
                     .into_iter()
                     .flat_map(|errors| errors.iter().map(|error| error as &dyn std::error::Error)),
             )
@@ -144,15 +142,15 @@ impl<'a> UnitErrorDescription<'a> {
     }
 }
 
-#[derive(Clone, Copy, Debug, Error)]
+#[derive(Clone, Debug, Error)]
 struct UnitAbortDescription {
-    status: AbortStatus,
+    description: AbortDescription,
     leaked: bool,
 }
 
 impl fmt::Display for UnitAbortDescription {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "process {}", display_abort_status(self.status))?;
+        write!(f, "process {}", self.description)?;
         if self.leaked {
             write!(f, ", and also leaked handles")?;
         }
@@ -209,10 +207,10 @@ impl<'a> TestOutputErrorSlice<'a> {
             }
         }
 
-        if let Some(stdout) = stdout {
-            if let Some(stdout_subslice) = heuristic_should_panic(stdout) {
-                return Some(TestOutputErrorSlice::ShouldPanic { stdout_subslice });
-            }
+        if let Some(stdout) = stdout
+            && let Some(stdout_subslice) = heuristic_should_panic(stdout)
+        {
+            return Some(TestOutputErrorSlice::ShouldPanic { stdout_subslice });
         }
 
         None
@@ -309,10 +307,10 @@ fn heuristic_panic_message(stderr: &[u8]) -> Option<ByteSubslice<'_>> {
     // result-based test failures.
     let mut start = panicked_at_match.start();
     let prefix = stderr[..start].trim_end_with(|c| c == '\n' || c == '\r');
-    if let Some(prev_line_start) = prefix.rfind("\n") {
-        if prefix[prev_line_start..].starts_with_str("\nError:") {
-            start = prev_line_start + 1;
-        }
+    if let Some(prev_line_start) = prefix.rfind("\n")
+        && prefix[prev_line_start..].starts_with_str("\nError:")
+    {
+        start = prev_line_start + 1;
     }
 
     // TODO: this grabs too much -- it is possible that destructors print out further messages so we
