@@ -7,10 +7,9 @@
 
 use super::{
     FinalStatusLevel, MaxProgressRunning, StatusLevel, TestOutputDisplay,
-    displayer::{DisplayReporter, DisplayReporterBuilder, StatusLevels},
+    displayer::{DisplayReporter, DisplayReporterBuilder, ShowTerminalProgress, StatusLevels},
 };
 use crate::{
-    cargo_config::CargoConfigs,
     config::core::EvaluatableProfile,
     errors::WriteEventError,
     list::TestList,
@@ -18,19 +17,20 @@ use crate::{
         aggregator::EventAggregator, displayer::ShowProgress, events::*,
         structured::StructuredReporter,
     },
+    write_str::WriteStr,
 };
 
-/// Standard error destination for the reporter.
+/// Output destination for the reporter.
 ///
 /// This is usually a terminal, but can be an in-memory buffer for tests.
-pub enum ReporterStderr<'a> {
+pub enum ReporterOutput<'a> {
     /// Produce output on the (possibly piped) terminal.
     ///
     /// If the terminal isn't piped, produce output to a progress bar.
     Terminal,
 
     /// Write output to a buffer.
-    Buffer(&'a mut String),
+    Writer(&'a mut (dyn WriteStr + Send)),
 }
 
 /// Test reporter builder.
@@ -126,8 +126,8 @@ impl ReporterBuilder {
         &self,
         test_list: &TestList,
         profile: &EvaluatableProfile<'a>,
-        cargo_configs: &CargoConfigs,
-        output: ReporterStderr<'a>,
+        show_term_progress: ShowTerminalProgress,
+        output: ReporterOutput<'a>,
         structured_reporter: StructuredReporter<'a>,
     ) -> Reporter<'a> {
         let aggregator = EventAggregator::new(test_list.mode(), profile);
@@ -153,8 +153,9 @@ impl ReporterBuilder {
             show_progress: self.show_progress,
             no_output_indent: self.no_output_indent,
             max_progress_running: self.max_progress_running,
+            show_term_progress,
         }
-        .build(cargo_configs, output);
+        .build(output);
 
         Reporter {
             display_reporter,
