@@ -16,7 +16,8 @@ use crate::{
     input::{InputEvent, InputHandler},
     list::{TestInstance, TestInstanceId, TestList},
     reporter::events::{
-        CancelReason, ExecuteStatus, ExecutionStatuses, FinalRunStats, InfoResponse, ReporterEvent,
+        CancelReason, ChildExecutionOutputDescription, ExecuteStatus, ExecutionResultDescription,
+        ExecutionStatuses, FailureDescription, FinalRunStats, InfoResponse, ReporterEvent,
         RunFinishedStats, RunStats, StressIndex, StressProgress, StressRunStats, TestEvent,
         TestEventKind,
     },
@@ -600,20 +601,20 @@ where
 
                 // Fire the setup-script-done probe, extracting the exit code
                 // from the result if available.
-                let exit_code = match status.result {
-                    crate::reporter::events::ExecutionResult::Fail {
-                        failure_status: crate::reporter::events::FailureStatus::ExitCode(code),
+                let exit_code = match &status.result {
+                    ExecutionResultDescription::Fail {
+                        failure: FailureDescription::ExitCode { code },
                         ..
-                    } => Some(code),
+                    } => Some(*code),
                     _ => None,
                 };
 
-                // Extract stdout and stderr lengths from the output
+                // Extract stdout and stderr lengths from the output.
                 let (stdout_len, stderr_len) = match &status.output {
-                    crate::test_output::ChildExecutionOutput::Output { output, .. } => {
+                    ChildExecutionOutputDescription::Output { output, .. } => {
                         output.stdout_stderr_len()
                     }
-                    crate::test_output::ChildExecutionOutput::StartError(_) => (None, None),
+                    ChildExecutionOutputDescription::StartError(_) => (None, None),
                 };
 
                 crate::fire_usdt!(UsdtSetupScriptDone {
